# -*- coding: utf-8 -*-
"""
Created on Thu Oct 20 17:44:11 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""


###########################################################################
#::: import standard/community packages
########################################################################### 
import os
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages


###########################################################################
#::: import my custom packages
########################################################################### 
from mytools import lightcurve_tools
from mytools.utils import mystr


###########################################################################
#::: import sub-packages
########################################################################### 
#import stacked_images
#from astropy.stats import LombScargle
#from scipy.signal import lombscargle
#import lightcurve_tools, get_scatter_color






def plot(dic, dic_nb, outdir, fieldname, obj_id, ngts_version, dt=0.01, show_plot=False):
    figs = {}
    figs[0] = plot_phasecurve_1_siderial_day( dic )
    figs[1] = plot_detrending_steps_evaluation( dic, dic_nb, dt )
    figs[2] = plot_neighbours_phasecurve_and_location( fieldname, ngts_version, dic, dic_nb, dt )
    figs[3] = plot_hjd_curves( dic, dic_nb )
    save_pdf( figs, outdir, fieldname, obj_id, ngts_version, show_plot )
    
    
    
###########################################################################
#::: save all plots in one pdf per target object
###########################################################################   
def save_pdf( figs, outdir, fieldname, obj_id, ngts_version, show_plot ):
    outfilename = os.path.join( outdir, fieldname + '_' + obj_id + '_' + ngts_version + '_centroid_appendix.pdf' )    
    with PdfPages( outfilename ) as pdf:
        for ind in figs: 
            pdf.savefig( figs[ind] )
        print 'Plots saved as ' + outfilename
        
    if show_plot == False: plt.close('all')
   


def plot_detrending_steps_evaluation( dic, dic_nb, dt ):   
    fig, axes = plt.subplots(1,2,figsize=(16,16))
    offset = 0.02
    xtext = -0.25    
    ytext = 0.01
    
    #::: phasefolded curves
    centdx = dic['CENTDX']
    centdy = dic['CENTDY']
    plot_phasecurves(dic, dt, centdx, centdy, color='g', axes=axes, offset=0*offset )
    axes[0].text( xtext, ytext-0*offset, 'target (raw)')
    
    centdx = dic['CENTDX_f']
    centdy = dic['CENTDY_f']
    plot_phasecurves( dic, dt, centdx, centdy, color='orange',  axes=axes, offset=1*offset )
    axes[0].text( xtext, ytext-1*offset, 'target (flattened externally)')
        
#    centdx = np.nanmean( dic_nb['CENTDX'], axis=0 )
#    centdy = np.nanmean( dic_nb['CENTDY'], axis=0 )
#    plot_phasecurves( dic, dt, centdx, centdy, color='b',  axes=axes, offset=2*offset )
#    axes[0].text( xtext, ytext-2*offset, 'neighbours (mean of all)')

#    centdx = dic['CENTDX_f'] - np.nanmean( dic_nb['CENTDX'], axis=0 )
#    centdy = dic['CENTDY_f'] - np.nanmean( dic_nb['CENTDY'], axis=0 )
#    plot_phasecurves( dic, dt, centdx, centdy, color='orange',  axes=axes, offset=3*offset )
#    axes[0].text( xtext, ytext-3*offset, 'target - neighbours (mean of all)')
    
    centdx = dic_nb['CENTDX_ref_mean']
    centdy = dic_nb['CENTDY_ref_mean']
    plot_phasecurves( dic, dt, centdx, centdy, color='b',  axes=axes, offset=2*offset )
    axes[0].text( xtext, ytext-2*offset, 'reference stars (mean of best fit)')
    
    centdx = dic['CENTDX_fd']
    centdy = dic['CENTDY_fd']
    plot_phasecurves( dic, dt, centdx, centdy, color='orange',  axes=axes, offset=3*offset )
    axes[0].text( xtext, ytext-3*offset, 'target (flattened and detrended, best fit)')
    
    centdx = dic['CENTDX_fda']
    centdy = dic['CENTDY_fda']
    plot_phasecurves( dic, dt, centdx, centdy, color='orange',  axes=axes, offset=4*offset )
    axes[0].text( xtext, ytext-4*offset, 'target (flattened, detrended and 1 day siderial airmass correction)')
    
    axes[0].set( xlim=[-0.25,0.75], ylim=[-0.02-4*offset,0.02] )
    axes[1].set( xlim=[-0.25,0.75], ylim=[-0.02-4*offset,0.02] )
    
    gap = dic['WIDTH'] / dic['PERIOD']
    for i in [0,1]:
        axes[i].axvspan( 0-gap/2., 0+gap/2., color='lightgrey', alpha=0.5 )
        axes[i].axvspan( 0.5-gap/2., 0.5+gap/2., color='lightgrey', alpha=0.5 )
    
    
    plt.tight_layout()
    return fig
    


def plot_hjd_curves( dic, dic_nb ):
    #::: set y offsets
    offset = 0.1
    
    #::: set plotting range
    N_points = len(dic['HJD_BIN'])
#    N_points = 1000
    
    #::: set x-axis
    x = np.arange(N_points)
#    x = dic['HJD_BIN'][slice(None)]
    
#    print '***********************************'
#    print x
#    print dic_nb['CENTDX_ref_mean_BIN'][slice(None)]
#    print '***********************************'
#    print x.shape
#    print dic_nb['CENTDX_ref_mean_BIN'][slice(None)].shape
#    print '***********************************'
#    print dic['SYSREM_FLUX3'].shape
#    print dic_nb['CENTDX'].shape
#    print dic_nb['CENTDY'].shape
    
    
    #::: set scatter color
    c = dic['COLOR_BIN']
    cmap = 'jet'
    
    #::: plot
    fig, axes = plt.subplots(4,1, sharex=True, sharey=False, figsize=(100,16))    
    texts = ['raw','reference stars','flattened + detrended','siderial day airmass correction', 'result']    

    ax = axes[0]
    ax.scatter( x, dic['SYSREM_FLUX3_BIN'][slice(None)], c=c, rasterized=True, cmap=cmap )
    ax.set( ylabel='FLUX (BINNED)' )    
    
    ax = axes[1]
    ax.scatter( x, dic['CENTDX_f_BIN'][slice(None)], c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.scatter( x, dic_nb['CENTDX_ref_mean_BIN'][slice(None)] - offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.scatter( x, dic['CENTDX_fd_BIN'][slice(None)] - 2*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    for i, text in enumerate( texts ): 
        ax.text( x[0], -i*offset, text )
        ax.axhline( -i*offset, color='k' )
    if 'poly_CENTDX_BIN' in dic:
        ax.scatter( x, dic['poly_CENTDX_BIN'][slice(None)] - 3*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    else:
        ax.scatter( x, dic['ma_CENTDX_BIN'][slice(None)] - 3*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.scatter( x, dic['CENTDX_fda_BIN'][slice(None)] - 4*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.set( ylim=[-0.1-4*offset,0.1], ylabel='CENTDX (BINNED)' )
        
    ax = axes[2]
    ax.scatter( x, dic['CENTDY_f_BIN'][slice(None)], c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.scatter( x, dic_nb['CENTDY_ref_mean_BIN'][slice(None)] - offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.scatter( x, dic['CENTDY_fd_BIN'][slice(None)] - 2*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    for i, text in enumerate( texts ): 
        ax.text( x[0], -i*offset, text )
        ax.axhline( -i*offset, color='k' )
    if 'poly_CENTDY_BIN' in dic:
        ax.scatter( x, dic['poly_CENTDY_BIN'][slice(None)] - 3*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    else:
        ax.scatter( x, dic['ma_CENTDY_BIN'][slice(None)] - 3*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.scatter( x, dic['CENTDY_fda_BIN'][slice(None)] - 4*offset, c=c, rasterized=True, cmap=cmap, vmin=-1, vmax=1 )
    ax.set( ylim=[-0.1-4*offset,0.1], ylabel='CENTDY (BINNED)' )

    ax = axes[3]
    ax.scatter( x, dic['AIRMASS_BIN'][slice(None)], c=c, rasterized=True, cmap=cmap )
    ax.set( ylim=[1.,2.], ylabel='AIRMASS (BINNED)' )
    
#    ax = axes[4]
#    ax.scatter( x, dic['COLOR_BIN'][slice(None)], c=c, rasterized=True, cmap=cmap )
#    ax.set( ylim=[-1.,1.], ylabel='COLOR_BIN' )
    
    ax.set( xlim=[x[0],x[-1]] )
    
    plt.tight_layout()
    return fig
    
 
 
def plot_phasecurve_1_siderial_day( dic ):
    '''
    1 mean siderial day = 
    ( 23 + 56/60. + 4.0916/3600. ) / 24. = 0.9972695787 days
    see e.g. https://en.wikipedia.org/wiki/Sidereal_time
    
    Note: dic['poly_CENTDX'] is a function!
    '''

    #::: show airmass as proof of concept for the siderial day phase folding
#    fig, axes = plt.subplots(1,2,figsize=(16,8))
#    
#    axes[0].scatter( dic['HJD_PHASE_1sidday'], dic['COLOR_PHASE_1sidday'], c=dic['COLOR_PHASE_1sidday'], rasterized=True, cmap='jet', vmin=-1, vmax=1)
#    axes[0].set( ylim=[-1.,1.] )
#
#    axes[1].scatter( dic['HJD_PHASE_1sidday'], dic['AIRMASS_PHASE_1sidday'], c=dic['COLOR_PHASE_1sidday'], rasterized=True, cmap='jet', vmin=-1, vmax=1)
#    axes[1].set( ylim=[1.,2.] )


    #::: show FLUX and CENTDXY
    fig, axes = plt.subplots(3,1,figsize=(8,6), sharex=True)
#    axes[0].scatter( dic['PHI'][::10], dic['CENTDX_fd'][::10], c=dic['COLOR'][::10], rasterized=True, cmap='jet' )
    axes[0].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDX_fd_PHASE_1sidday'], c=dic['COLOR_PHASE_1sidday'], rasterized=True, cmap='jet', vmin=-1, vmax=1 )
#    axes[0].errorbar( dic['HJD_PHASE_1sidday'], dic['CENTDX_fd_PHASE_1sidday'], yerr=dic['CENTDX_fd_PHASE_1sidday_ERR'], fmt='.', color='k' )
    
    axes[1].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDY_fd_PHASE_1sidday'], c=dic['COLOR_PHASE_1sidday'], rasterized=True, cmap='jet', vmin=-1, vmax=1 )
#    axes[1].errorbar( dic['HJD_PHASE_1sidday'], dic['CENTDY_fd_PHASE_1sidday'], yerr=dic['CENTDY_fd_PHASE_1sidday_ERR'], fmt='.', color='k' )

    axes[2].scatter( dic['HJD_PHASE_1sidday'], dic['SYSREM_FLUX3_PHASE_1sidday'], c=dic['COLOR_PHASE_1sidday'], rasterized=True, cmap='jet', vmin=-1, vmax=1 )
#    axes[2].errorbar( dic['HJD_PHASE_1sidday'], dic['SYSREM_FLUX3_PHASE_1sidday'], yerr=dic['SYSREM_FLUX3_PHASE_1sidday_ERR'], fmt='.', color='k' )
    
    
    #::: show FLUX and CENTDXY trends / polyfits
    if 'polyfct_CENTDX' in dic:
        axes[0].plot( dic['HJD_PHASE_1sidday'], dic['polyfct_CENTDX'](dic['HJD_PHASE_1sidday']), 'r-' )
        axes[0].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDX_fd_PHASE_1sidday'] - dic['polyfct_CENTDX'](dic['HJD_PHASE_1sidday']), c='r', rasterized=True )
        axes[0].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDX_fda_PHASE_1sidday'], c='g', rasterized=True )
    
        axes[1].plot( dic['HJD_PHASE_1sidday'], dic['polyfct_CENTDY'](dic['HJD_PHASE_1sidday']), 'r-' )
        axes[1].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDY_fd_PHASE_1sidday'] - dic['polyfct_CENTDY'](dic['HJD_PHASE_1sidday']), c='r', rasterized=True )
        axes[1].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDY_fda_PHASE_1sidday'], c='g', rasterized=True )

    else:
        axes[0].plot( dic['HJD_PHASE_1sidday'], dic['ma_CENTDX_PHASE_1sidday'], 'r-' )
        axes[0].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDX_fd_PHASE_1sidday'] - dic['ma_CENTDX_PHASE_1sidday'], c='r', rasterized=True )
        axes[0].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDX_fda_PHASE_1sidday'], c='g', rasterized=True )
    
        axes[1].plot( dic['HJD_PHASE_1sidday'], dic['ma_CENTDY_PHASE_1sidday'], 'r-' )
        axes[1].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDY_fd_PHASE_1sidday'] - dic['ma_CENTDY_PHASE_1sidday'], c='r', rasterized=True )
        axes[1].scatter( dic['HJD_PHASE_1sidday'], dic['CENTDY_fda_PHASE_1sidday'], c='g', rasterized=True )
    
    
    axes[0].set( ylim=[-0.005,0.005] )
    axes[1].set( ylim=[-0.005,0.005] )
    axes[2].set( ylim=[0.995,1.005] )
    
    plt.tight_layout()
    return fig

    
    
def plot_neighbours_phasecurve_and_location( fieldname, ngts_version, dic, dic_nb, dt ):
        
    N_nb = len(dic_nb['OBJ_ID'])
    if N_nb > 20:
        N_nb = 20
    fig, axes = plt.subplots(N_nb, 5, figsize=(20,N_nb*4))

    for i in range(N_nb):
        centdx = dic_nb['CENTDX'][i,:]
        centdy = dic_nb['CENTDY'][i,:]
        plot_phasecurves_extended( fieldname, ngts_version, dic, dic_nb, i, dt, centdx, centdy, axes=axes[i,:])
        
    plt.tight_layout()
    return fig
        


def plot_phasecurves( dic, dt, centdx, centdy, title=None, color='b', axes=None, offset=None ):
    
    hjd_phase, centdx_c_phase, centdx_c_phase_err, _, _ = lightcurve_tools.phase_fold( dic['HJD'], centdx - np.nanmean(centdx), dic['PERIOD'], dic['EPOCH'], dt = dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)
    hjd_phase, centdy_c_phase, centdy_c_phase_err, _, _ = lightcurve_tools.phase_fold( dic['HJD'], centdy - np.nanmean(centdy), dic['PERIOD'], dic['EPOCH'], dt = dt, ferr_type='meansig', ferr_style='sem', sigmaclip=True)

    if offset is not None:
        centdx_c_phase -= offset
        centdy_c_phase -= offset

    if axes is None: 
        fig, axes = plt.subplots(1,2, sharex=True, sharey=True, figsize=(12,4))

    axes[0].errorbar( hjd_phase, centdx_c_phase, yerr=centdx_c_phase_err, fmt='o', color=color, rasterized=True ) #, color='darkgrey')
    axes[0].set_ylabel('CENTDX (in pixel)')
#        axes[0].set_ylim([ np.min(centdx_c_phase - centdx_c_phase_err), np.max(centdx_c_phase + centdx_c_phase_err) ])
    
    axes[1].errorbar( hjd_phase, centdy_c_phase, yerr=centdy_c_phase_err, fmt='o', color=color, rasterized=True ) #, color='darkgrey')
    axes[1].set_ylabel('CENTDY (in pixel)')
#        axes[1].set_ylim([ np.min(centdy_c_phase - centdy_c_phase_err), np.max(centdy_c_phase + centdy_c_phase_err) ])

    axes[0].set( xlim=[-0.25,0.75], ylim=[-0.02,0.02] )
    axes[1].set( xlim=[-0.25,0.75], ylim=[-0.02,0.02] )
    
    if title is not None:
        plt.suptitle( title )
    
    

def plot_phasecurves_extended( fieldname, ngts_version, dic, dic_nb, i, dt, centdx, centdy, color='b', axes=None ):
    
    if axes is None:
        fig, axes = plt.subplots(1,5, figsize=(20,4))
    
    plot_phasecurves( dic, dt, centdx, centdy, color='b', axes=axes )
    
    axes[2].plot( dic['CCDX'][0], dic['CCDY'][0], 'bo', ms=12 )
    axes[2].plot( dic_nb['CCDX_0'], dic_nb['CCDY_0'], 'k.' )
    axes[2].plot( dic_nb['CCDX_0'][i], dic_nb['CCDY_0'][i], 'ro', ms=12 )
    axes[2].set( xlim=[ dic['CCDX'][0]-150, dic['CCDX'][0]+150 ], ylim=[ dic['CCDY'][0]-150, dic['CCDY'][0]+150 ] )
    
    #TODO: astropy version errors
    #stacked_images.plot(fieldname, ngts_version, dic_nb['CCDX_0'][i], dic_nb['CCDY_0'][i], r=15, ax=axes[3], show_apt=True, show_cbar=True)
    
    plot_neighbour_info_text(axes[4], dic, dic_nb, i)
        
        


###########################################################################
#::: plot info page
###########################################################################  
def plot_neighbour_info_text(ax, dic, dic_nb, i):
        
    ax.set_xlim([0,1])
    ax.set_ylim([0,1])
    ax.axis('off')
    ax.text(0,1.0,'OBJ_ID: '+dic_nb['OBJ_ID'][i])
    ax.text(0,0.9,'FLUX: '+mystr(dic_nb['FLUX_MEAN'][i],2))
    ax.text(0,0.8,'CCDX_0: '+mystr(dic_nb['CCDX_0'][i],2))
    ax.text(0,0.7,'CCDY_0: '+mystr(dic_nb['CCDY_0'][i],2))
    ax.text(0,0.6,'CCD distance: '+mystr(np.sqrt( (dic['CCDX'][0] - dic_nb['CCDX_0'][i])**2 + (dic['CCDY'][0] - dic_nb['CCDY_0'][i])**2 ),2))
    ax.text(0,0.5,'CCD_X distance: '+mystr(( dic['CCDX'][0] - dic_nb['CCDX_0'][i] ),2))
    ax.text(0,0.4,'CCD_Y distance: '+mystr(( dic['CCDY'][0] - dic_nb['CCDX_0'][i] ),2))
    try:
        ax.text(0,0.3,'B-V color: '+mystr(dic_nb['B-V'][i],2))
        ax.text(0,0.2,'B-V color difference: '+mystr(dic['B-V'] - dic_nb['B-V'][i],2))
        ax.text(0,0.1,'V Mag: '+mystr(dic_nb['Vmag'][i],2))
    except:
        pass
    ax.text(0,0.0,'Corr Coeff X / Y: '+mystr(dic_nb['corrcoeff_x'][i],2) + ' / ' + mystr(dic_nb['corrcoeff_x'][i],2))
        
    



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 14 23:26:13 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""

###########################################################################
#::: import standard/community packages
########################################################################### 
import sys, os, warnings
import numpy as np
from itertools import izip
from centroiding import centroid


###########################################################################
#::: import my custom packages
########################################################################### 
from ngtsio import ngtsio




###########################################################################
#::: canvas or bls run (for all individual canvas targets)
###########################################################################              
def evaluate_all_fields_canvas(key, ngts_version, dt, cut=False):
    """
    EVALUATE ALL FIELDS' CANVAS OBJECTS
    """
    
    #::: create outdir and flagfile
    outdir = os.path.join( 'output', key+'_'+ngts_version+'_dt='+str(dt)+'_cut='+str(cut), '' )
    if not os.path.exists(outdir): os.makedirs(outdir)
    flagfile = initialize_flagfile(outdir)       
            
    #::: read in fieldnames
    fieldnames = np.genfromtxt('input/FIELD_LIST.txt', dtype=None)
    if not isinstance(fieldnames, np.ndarray): fieldnames = [fieldnames]
        
    #::: proceed in loop for each field
    for fieldname in fieldnames:
        print '###########################################################################'
        print fieldname
        print '###########################################################################'
        canvasbls_run(key, fieldname, ngts_version, dt, cut, outdir, flagfile)
#        except:
#            warnings.warn('Skipped because of problems.')   
            
            
            
def canvasbls_run(key, fieldname, ngts_version, dt, cut, outdir, flagfile):
    '''
    key: 'canvas' or 'bls'
    '''
    #:::
    if key=='canvas':
        depth_key = 'CANVAS_DEPTH'
    else:
        depth_key = 'DEPTH'
    
    #::: load canvas obj_id list
    objdic = ngtsio.get( fieldname, ngts_version, ['OBJ_ID', depth_key, 'FLUX_MEAN', 'CCD_X', 'CCD_Y'], obj_id=key, silent=True)
    '''
    (note that 'CCD_X' denotes the mean location, while 'CCDX' is the location per exposure)
    '''
    
    if objdic is None:
        warnings.warn(fieldname+' not existant on this server.')
    else:
        
        #::: depth cut-off
        if cut:
            depth_cut = -0.01 #1%
            flux_cut = 500. #min 500 ADU/s
            ccd_cut = 50. #at least 50 pixels away from the edge
            obj_ids = objdic['OBJ_ID'][ (objdic[depth_key] < depth_cut) & \
                                        (objdic['FLUX_MEAN'] > flux_cut) & \
                                        (objdic['CCD_X'] > ccd_cut) & \
                                        (objdic['CCD_X'] < (2048.-ccd_cut)) & \
                                        (objdic['CCD_Y'] > ccd_cut) & \
                                        (objdic['CCD_Y'] < (2048.-ccd_cut)) ]
        else:
            obj_ids = objdic['OBJ_ID']
        
        #::: run for all individually
        for i, obj_id in enumerate( obj_ids ):
            print '\n'
            print '###########################################################################'
            print i, '/', len(obj_ids), ',', obj_id
            print '###########################################################################'
            
            try:
                C = centroid(fieldname, obj_id, ngts_version=ngts_version, dt=dt, outdir=outdir, flagfile=flagfile)
                C.run()
            except:
                with open( flagfile, 'a+') as f: 
                    f.write('# '+fieldname+'\t'+obj_id+'\t'+'(broke)\n') 
                warnings.warn('Skipped because of problems.')   



###########################################################################
#::: candidate shortlist run
###########################################################################         
def evaluate_candidate_shortlist( fname, dt=0.005, outdir=None, mode='skipfiles', **kwargs ):
    """
    EVALUATE ALL THE CANDIDATES E.G. FROM THE GOOGLE SPREADSHEET
    input must be an ascii (e.g. txt) file with four columns:
    fieldname obj_id ngts_version source
    source: BLS or CANVAS
    """
    
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    #::: load candidate list
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    data = np.genfromtxt(fname, names=['fieldnames', 'obj_ids', 'ngts_versions', 'source'], dtype=None) #dtype=['|S11','|S6',None])
    N_obj = len(data['obj_ids'])


    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    #::: select output directory
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    if outdir is None:
        outdir = os.path.join( 'output', 'candidate_shortlists', os.path.basename(os.path.splitext(fname)[0]), 'dt='+str(dt) ) #no ending slash here (the ending slash will be added later)
    
    #if the dir already exists and mode==newdir, create it with suffix  
    if ( os.path.exists(outdir) & (mode=='newdir') ):
        
        i = 2
        outdir += '_' + str(i) #outdir_2
        while os.path.exists(outdir):
            i += 1
            outdir = outdir[0:-1] + str(i) #outdir_3, outdir_4, ...
  
        outdir = os.path.join( outdir, '' )
        os.makedirs(outdir)   
        
    #if the dir already exists and mode==skipfiles or replacefiles, do nothing 
    elif ( os.path.exists(outdir) & ( mode in ('skipfiles', 'replacefiles') ) ):
        pass
    
    #if the dir does not exists, create it
    elif ~os.path.exists(outdir):
        outdir = os.path.join( outdir, '' )
        os.makedirs(outdir)   
    

       
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    #::: if there's already a flagfile, rename it into flagfile_old
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    if (mode in ('skipfiles', 'replacefiles')) & (os.path.exists( os.path.join(outdir,'flagfile.txt') )):
        flagfile = os.path.join( outdir, 'flagfile.txt' )
        flagfile_old = os.path.join( outdir, 'flagfile_old.txt' )
        os.rename(flagfile, flagfile_old) #rename the original flagfile to flagfile_old
    else:
        pass
    
    
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    #::: create flagfile
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    flagfile = initialize_flagfile(outdir)
            
            
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    #::: loop over all candidates
    #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    i = 0
    for fieldname, obj_id, ngts_version, source in izip(data['fieldnames'], data['obj_ids'], data['ngts_versions'], data['source']):
        #::: load obj_id as string with leading zeros (for consistency in filenames)        
        obj_id = str(obj_id).zfill(6)
        i += 1
        print '###########################################################################'
        print str(i)+'/'+str(N_obj), fieldname, obj_id, ngts_version, source
        
        #check in flagfile_old if obj was already evaluated
        #if yes, copy it's results from flagfile_old
        #and write it into the new flagfile 
        oldline = None
        if (mode=='skipfiles') & (os.path.exists( os.path.join(outdir,'flagfile_old.txt') )):
            with open( flagfile_old, 'r' ) as of:
                for line in of:
                    if (fieldname in line) & (obj_id in line):
                        oldline = line
                        
        if oldline is not None:
            print "Skipped: object previously evaluated, flagfile entry already existant and mode=='skipfiles'."  
            with open( flagfile, 'a+') as f: 
                f.write( oldline ) 
        
        else:
            try:
                C = centroid(fieldname, obj_id, ngts_version, source=source, dt=dt, outdir=outdir, flagfile=flagfile, show_plot=False, **kwargs)
                C.run()
            except:
                with open( flagfile, 'a+') as f: 
                    f.write('#'+fieldname+'\t'+obj_id+'\t'+'(broke)\n') 
                print 'Skipped because of problems.'
        


###########################################################################
#::: initialize a flagfile
###########################################################################   
def initialize_flagfile(outdir):
    flagfile = os.path.join( outdir, 'flagfile.txt' )
    header = '#FIELDNAME' + '\t' +\
         'OBJ_ID' + '\t' +\
         'NGTS_VERSION' + '\t' +\
         'RA' + '\t' +\
         'DEC' + '\t' +\
         'CCDX_0' + '\t' +\
         'CCDY_0' + '\t' +\
         'FLUX_MEAN' + '\t' +\
         'CENTDX_fda_PHASE_RMSE' + '\t' +\
         'CENTDY_fda_PHASE_RMSE' + '\t' +\
         'CENTDX_fd_PHASE_RMSE' + '\t' +\
         'CENTDY_fd_PHASE_RMSE' + '\t' +\
         'CENTDX_f_PHASE_RMSE' + '\t' +\
         'CENTDY_f_PHASE_RMSE' + '\t' +\
         'CENTDX_PHASE_RMSE' + '\t' +\
         'CENTDY_PHASE_RMSE' + '\t' +\
         'RollCorrSNR_X' + '\t' +\
         'RollCorrSNR_Y' + '\t' +\
         'CrossCorrSNR_X' + '\t' +\
         'CrossCorrSNR_Y' + '\t' +\
         'Ttest_X' + '\t' +\
         'Ttest_Y' + '\t' +\
         'Binom_X' + '\t' +\
         'Binom_Y'
    with open( flagfile, 'w') as f:
        f.write(header+'\n')  
    return flagfile




###########################################################################
#::: MAIN
###########################################################################  
if __name__ == '__main__':
    if len(sys.argv) == 1:
        print 'Input file/command required.'
        
    if len(sys.argv) == 2:
        if sys.argv[1] in ['bls', 'canvas']:
            evaluate_all_fields_canvas(str(sys.argv[1]))
        else:
            evaluate_candidate_shortlist(str(sys.argv[1]), dt=0.005)
            evaluate_candidate_shortlist(str(sys.argv[1]), dt=0.0025)
            evaluate_candidate_shortlist(str(sys.argv[1]), dt=0.001)
            
    #sys.argv[2] is the mode 'newdir', skipfiles' or 'replacefiles'
    if len(sys.argv) == 3:
        if sys.argv[2] not in ('newdir', 'skipfiles', 'replacefiles'):
            warnings.warn("Aborted. Second argument has to be 'newdir', 'skipfiles', or 'replacefiles'.")
        else:
            if sys.argv[1] in ['bls', 'canvas']:
                evaluate_all_fields_canvas(str(sys.argv[1]))
            else:
                evaluate_candidate_shortlist(str(sys.argv[1]), dt=0.005, mode=sys.argv[2])
                evaluate_candidate_shortlist(str(sys.argv[1]), dt=0.0025, mode=sys.argv[2])
                evaluate_candidate_shortlist(str(sys.argv[1]), dt=0.001, mode=sys.argv[2])
            
    if len(sys.argv) == 5:
        'e.g. "bls TEST18 0.001 -0.01" means take BLS objects from TEST18, analyse for dt=0.001, and only select DEPTH<-0.01'
        if sys.argv[1] in ['bls', 'canvas']:
            evaluate_all_fields_canvas(str(sys.argv[1]), str(sys.argv[2]), float(sys.argv[3]), float(sys.argv[4]))
        else:
            raise ValueError('If passing three arguments, the first must be "bls" or "canvas", the second the desired dt value, and the third the desired depth cut-off.')
        
    



# -*- coding: utf-8 -*-
"""
Created on Thu Nov 10 17:02:38 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""

import numpy as np
#import matplotlib.pyplot as plt

from mytools.utils import mystr, deg2hmsdms
#from mytools.plot_utils import norm_scatter






def plot_target_info_text(ax, C):
    '''
    C   centroiding class (passed over as 'self')
    '''
    if 'DEPTH' not in C.dic: depth = '.'
    else: depth = mystr(np.abs(C.dic['DEPTH'])*1000.,2)
    if 'NUM_TRANSITS' not in C.dic: num_transits = '.'
    else: num_transits = mystr(C.dic['NUM_TRANSITS'],0)
    
    hmsdms = deg2hmsdms(C.dic['RA']*180/np.pi, C.dic['DEC']*180/np.pi)
    
    ax.set_xlim([0,1])
    ax.set_ylim([0,1])
    ax.axis('off')
    ax.text(0,1.0,C.fieldname)
    ax.text(0,0.9,C.dic['OBJ_ID'])
    ax.text(0,0.8,hmsdms)
    ax.text(0,0.7,'Flux: '+str(C.dic['FLUX_MEAN']))
    ax.text(0,0.6,'V-mag: '+mystr(C.dic['Vmag'],3))
    ax.text(0,0.5,'B-V color: '+mystr(C.dic['B-V'],3))
    
    ax.text(0,0.4,C.source + ', rank=' + str(C.bls_rank))
    ax.text(0,0.3,'Period (d): '+mystr(C.dic['PERIOD']/3600./24.,3))
    ax.text(0,0.2,'Epoch (s): '+mystr(C.dic['EPOCH'],2))
    ax.text(0,0.1,'Width (h): '+mystr(C.dic['WIDTH']/3600.,2))
    ax.text(0,0.0,'Depth (mmag): '+depth+', Nr. of transits: '+num_transits)




def plot_neighbour_info_text(ax, C, j):
        
    hmsdms = deg2hmsdms(C.dic_blends['RA'][j]*180/np.pi, C.dic_blends['DEC'][j]*180/np.pi)
    
    ax.set_xlim([0,1])
    ax.set_ylim([0,1])
    ax.axis('off')
    ax.text(0,0.9,C.dic_blends['OBJ_ID'][j])
    ax.text(0,0.8,hmsdms)
    ax.text(0,0.7,'Flux: '+mystr(C.dic_blends['FLUX_MEAN'][j],2))
    ax.text(0,0.6,'V Mag: '+mystr(C.dic_blends['Vmag'][j],2))
    ax.text(0,0.5,'B-V color: '+mystr(C.dic_blends['B-V'][j],2))
    ax.text(0,0.4,r'$\Delta$CCD x: '+mystr(( C.dic['CCDX'][0] - C.dic_blends['CCDX_0'][j] ),2))
    ax.text(0,0.3,r'$\Delta$CCD y: '+mystr(( C.dic['CCDY'][0] - C.dic_blends['CCDY_0'][j] ),2))

        





''' TODO '''

'''
def plot_detrending(self):
    ind = slice( None )
    
#            ind = np.where( self.dic['NIGHT'] == date )    
#            
#            #::: plot neighbours
#            N = len( self.dic_nb['OBJ_ID'] )
#            N_cols = 4
#            N_rows = int(np.ceil(1.*N/N_cols))                
#            fig, axes = plt.subplots( N_rows, N_cols, sharex=True, sharey=True, figsize=(4*N_cols,4*N_rows))
#            for i,_ in enumerate( self.dic_nb['OBJ_ID'] ):
#                ind_ax = np.unravel_index(i, (N_rows, N_cols))
#                axes[ ind_ax ].scatter( self.dic_nb['CENTDX'][i][ind], self.dic_nb['CENTDY'][i][ind], c=ind, rasterized=True)#, label='OBJID='+str(self.dic_nb['OBJ_ID'][i])+'\nFLUX='+str(self.dic_nb['FLUX_MEAN'][i]) + '\nCCDX=' + str(self.dic_nb['CCDX'][i,0]) + '\nCCDY=' + str(self.dic_nb['CCDY'][i,0]) )
#                axes[ ind_ax ].legend(loc='upper left', framealpha=0.5, scatterpoints=1)
#                axes[ ind_ax ].axis('equal')
#            plt.suptitle( date, fontsize=16 )
#            plt.tight_layout()
#            plt.subplots_adjust(top=0.95)
#    #        pdf.savefig()
#    #        plt.close()
    

    
    #::: plot object CENTDXY results
    fig, axes = plt.subplots( 1, 3, sharex=True, sharey=True, figsize=(4*3,4*1))
    
    xkeys = ['CENTDX', 'CENTDX_detrended_mean', 'CENTDX_detrended_median']
    ykeys = ['CENTDY', 'CENTDY_detrended_mean', 'CENTDY_detrended_median']
    titles = ['before local detrending','after local detrending (mean)','after local detrending (median)']
    for i, (xkey, ykey, title) in enumerate( zip( xkeys, ykeys, titles ) ):
        ax = axes[i]
        ax, sc = norm_scatter(ax, self.dic[xkey][ind], self.dic[ykey][ind], self.dic['tr_flag'][ind] )
        ax.set( xlim=[-0.1,0.1], ylim=[-0.1,0.1], title=title )
        ax.axis('equal')
    
    
#        axes[1,0].plot( self.df_gb_date['CENTDX'].std() )
#        
#        axes[1,1].plot( self.df_gb_date['CENTDY'].std() )
#        
#        axes[2,0].plot( self.df_gb_date['CENTDX_dmedc'].std() )
#        
#        axes[2,1].plot( self.df_gb_date['CENTDY_dmedc'].std() )
    
    plt.tight_layout()
    
    self.fig_detrending = fig



def plot_rainplot_summary(self):
    
    for chosendf, bin_style in izip( [self.df_gb_tr_flag, self.bindf_gb_tr_flag], ['unbinned', 'binned' ]):
    
        fig, ax = plt.subplots( 1, 1 )
        fig.suptitle(bin_style)
        colors = ['grey','k','r','goldenrod']
        i = 0
        for tr_flag, gdata in chosendf:
            ax.plot( gdata['CENTDX_dmedc'], gdata['CENTDY_dmedc'], 'k.', c=colors[i%4], alpha=0.33, rasterized=True, zorder=-1 )
            ax.errorbar( gdata['CENTDX_dmedc'].mean(), gdata['CENTDY_dmedc'].mean(), xerr=gdata['CENTDX_dmedc'].std(), yerr=gdata['CENTDY_dmedc'].std(), color=colors[i%4], lw=3, zorder=1 )             
            i += 1
        plt.axis('square')        
        plt.xticks(rotation=45)
            
        if bin_style == 'binned': self.fig1a = fig
        if bin_style == 'unbinned': self.fig2a = fig    
    


def plot_rainplot_per_night(self):
    
    for chosendf, bin_style in izip( [self.df_gb_date_tr_flag, self.bindf_gb_date_tr_flag], ['unbinned', 'binned' ]):

        i = 0
        N_dates = len( chosendf )/4
        
        fig, axes = plt.subplots( np.int( N_dates/4. )+1, 4, sharex=True, sharey=True, figsize=(4*3,N_dates))
        fig.suptitle(bin_style)
        colors = ['grey','k','r','goldenrod']
        for (date, tr_flag), gdata in chosendf:
            if i%4 == 0:        
                axind = np.unravel_index( i/4, (axes.shape[0],axes.shape[1]) )
                ax = axes[axind]
                ax.set_title(date)
                plt.xticks(rotation=45)
#                    ax.set_xlim([-0.1, 0.1])
#                    ax.set_ylim([-0.1, 0.1])
#                    plt.axis('square')                
#                    plt.axis('equal') 
            ax.plot( gdata['CENTDX_dmedc'], gdata['CENTDY_dmedc'], 'k.', c=colors[i%4], alpha=0.33, rasterized=True, zorder=-1 )
            ax.errorbar( gdata['CENTDX_dmedc'].mean(), gdata['CENTDY_dmedc'].mean(), xerr=gdata['CENTDX_dmedc'].std(), yerr=gdata['CENTDY_dmedc'].std(), color=colors[i%4], lw=3, zorder=1 )    
            i += 1
            
        if bin_style == 'binned': self.fig1 = fig
        if bin_style == 'unbinned': self.fig2 = fig



def plot_detrending_over_time(self):
    
    fig, axes = plt.subplots( 5, 1, sharex=True, figsize=(8,16) )
    self.bindf.plot( kind='scatter', x='HJD', y='SYSREM_FLUX3', c='tr_flag', ax=axes[0] )
    self.bindf.plot( kind='scatter', x='HJD', y='CENTDX_dmedc', c='tr_flag', ax=axes[1] )
    self.bindf.plot( kind='scatter', x='HJD', y='CENTDY_dmedc',  c='tr_flag', ax=axes[2] )
    self.bindf.plot( kind='scatter', x='HJD', y='CENTDX_dmedc_ERR',  c='tr_flag', ax=axes[3] )
    self.bindf.plot( kind='scatter', x='HJD', y='CENTDY_dmedc_ERR', c='tr_flag', ax=axes[4] )
    self.fig_detrending_over_time = fig
    

 
###########################################################################
#::: look at phase-folded lightcurve and centroid curve
###########################################################################        
def plot_phase_folded_curves(self):
    
    #::: detrended curves
    fig, axes = plt.subplots( 4, 1, sharex=True, figsize=(16,16) )
    
    axes[0].plot( self.dic['PHI'], self.dic['SYSREM_FLUX3']/np.nanmedian(self.dic['SYSREM_FLUX3']), 'k.', alpha=0.1, rasterized=True )
    axes[0].errorbar( self.dic['HJD_PHASE'], self.dic['SYSREM_FLUX3_PHASE'], yerr=self.dic['SYSREM_FLUX3_PHASE_ERR'], fmt='o', color='r', rasterized=True )
#        axes[0].errorbar( self.dic['HJD_PHASE'], self.dic['SYSREM_FLUX3_PHASEmedian'], yerr=self.dic['SYSREM_FLUX3_PHASEmedian_ERR'], fmt='o', color='grey', alpha=0.5, rasterized=True )
    axes[0].set_ylabel('FLUX')
    axes[0].set_ylim([ np.min(self.dic['SYSREM_FLUX3_PHASE']-self.dic['SYSREM_FLUX3_PHASE_ERR']), np.max(self.dic['SYSREM_FLUX3_PHASE']+self.dic['SYSREM_FLUX3_PHASE_ERR']) ])
    
    axes[1].plot( self.dic['PHI'], self.dic['CENTDX'], 'k.', alpha=0.1, rasterized=True )
    axes[1].errorbar( self.dic['HJD_PHASE'], self.dic['CENTDX_dmedc_PHASE'], yerr=self.dic['CENTDX_dmedc_PHASE_ERR'], fmt='o', rasterized=True ) #, color='darkgrey')
#        axes[1].errorbar( self.dic['HJD_PHASE'], self.dic['CENTDX_dmedc_PHASEmedian'], yerr=self.dic['CENTDX_dmedc_PHASEmedian_ERR'], fmt='o', color='grey', alpha=0.5, rasterized=True ) #, color='darkgrey')
    axes[1].set_ylabel('CENTDX (in pixel)')
    axes[1].set_ylim([ np.min(self.dic['CENTDX_dmedc_PHASE']-self.dic['CENTDX_dmedc_PHASE_ERR']), np.max(self.dic['CENTDX_dmedc_PHASE']+self.dic['CENTDX_dmedc_PHASE_ERR']) ])
    
    axes[2].plot( self.dic['PHI'], self.dic['CENTDY'], 'k.', alpha=0.1, rasterized=True )
    axes[2].errorbar( self.dic['HJD_PHASE'], self.dic['CENTDY_dmedc_PHASE'], yerr=self.dic['CENTDY_dmedc_PHASE_ERR'], fmt='o', rasterized=True ) #, color='darkgrey')
#        axes[2].errorbar( self.dic['HJD_PHASE'], self.dic['CENTDY_dmedc_PHASEmedian'], yerr=self.dic['CENTDY_dmedc_PHASEmedian_ERR'], fmt='o', color='grey', alpha=0.5, rasterized=True ) #, color='darkgrey')
    axes[2].set_ylabel('CENTDY (in pixel)')
    axes[2].set_xlabel('Phase')
    axes[2].set_ylim([ np.min(self.dic['CENTDY_dmedc_PHASE']-self.dic['CENTDY_dmedc_PHASE_ERR']), np.max(self.dic['CENTDY_dmedc_PHASE']+self.dic['CENTDY_dmedc_PHASE_ERR']) ])
    axes[2].set_xlim([-0.25,0.75])
    
    axes[3].plot( self.dic['HJD_PHASE'], self.dic['N_PHASE'], 'go', rasterized=True ) #, color='darkgrey')
    axes[3].set_ylabel('Nr of exposures')
    axes[3].set_xlabel('Phase')
    axes[3].set_xlim([-0.25,0.75])
    
    plt.tight_layout()
    self.fig_phasefold = fig      
    
    


###########################################################################
#::: plot info page
###########################################################################  
def plot_info_page(self):
    #::: plot object
    fig = plt.figure(figsize=(16,4))
    gs = gridspec.GridSpec(1, 4)
    
    #::: plot locations on CCD    
    ax = plt.subplot(gs[0, 0])
    label = 'Flux ' + str(self.flux_min) + '-' + str(self.flux_max) + ', ' + str(self.pixel_radius) + ' px'
    ax.plot( self.dic_nb['CCDX_0'], self.dic_nb['CCDY_0'], 'k.', rasterized=True, label=label )
    ax.plot( self.dic['CCDX'][0], self.dic['CCDY'][0], 'r+', rasterized=True ) 
    ax.add_patch( patches.Rectangle(
                    (self.dic['CCDX'][0]-self.pixel_radius, self.dic['CCDY'][0]-self.pixel_radius),
                    2*self.pixel_radius,
                    2*self.pixel_radius,
                    fill=False, color='r') )
#    ax.axis('equal')
    ax.set_xlim([0,2048])  
    ax.set_ylim([0,2048]) 
    ax.legend(loc='best', numpoints=1)
    
    
    #::: plot lightcurve
    ax = plt.subplot(gs[0, 1:3])
    ax.plot( self.dic['HJD'], self.dic['SYSREM_FLUX3'], 'k.', rasterized=True )
    ax.set(xlim=[np.min(self.dic['HJD']), np.max(self.dic['HJD'])])
    
    
    
    #::: plot info text
    ax = plt.subplot(gs[0, 3])
    self.plot_info_text(ax)    
    
    
    plt.tight_layout()
    
    self.fig_info_page = fig



def plot_info_text(self, ax):
    ax.set_xlim([0,1])
    ax.set_ylim([0,1])
    ax.axis('off')
    ra, dec = deg2HMS.deg2HMS(ra=self.dic['RA'], dec=self.dic['DEC'])
    ax.text(0,1.0,'OBJ_ID: '+self.dic['OBJ_ID']+'\t'+self.source)
    ax.text(0,0.9,'FLUX: '+str(self.dic['FLUX_MEAN']))
    ax.text(0,0.8,'RA (deg): '+ra)
    ax.text(0,0.7,'DEC (deg): '+dec)
    ax.text(0,0.6,'PERIOD (s): '+mystr(self.dic['PERIOD'],2))
    ax.text(0,0.5,'PERIOD (d): '+mystr(self.dic['PERIOD']/3600./24.,2))
    ax.text(0,0.4,'Width (s): '+mystr(self.dic['WIDTH'],2))
    ax.text(0,0.3,'Width (h): '+mystr(self.dic['WIDTH']/3600.,2))
    ax.text(0,0.2,'EPOCH (s): '+mystr(self.dic['EPOCH'],2))
    ax.text(0,0.1,'Depth (mmag): '+mystr(np.abs(self.dic['DEPTH'])*1000.,2))
    ax.text(0,0.0,'Num Transits: '+mystr(self.dic['NUM_TRANSITS'],0))
    


def plot_stacked_image(self):
    
    self.fig_stacked_image = stacked_images.plot(self.fieldname, np.nanmean(self.dic['CCDX']), np.nanmean(self.dic['CCDY']), r=15) 
    
    
    


###########################################################################
#::: save all plots in one pdf per target object
###########################################################################   
def save_pdf(self):
    outfilename = os.path.join( self.outdir, self.fieldname + '_' + self.obj_id + '_centroid_analysis.pdf' )
    with PdfPages( outfilename ) as pdf:
#            pdf.savefig( self.fig3 ) #self.fig_phasefold
#            pdf.savefig( self.fig1a )
#            pdf.savefig( self.fig2a )            
#            pdf.savefig( self.fig1 )
#            pdf.savefig( self.fig2 )
        pdf.savefig( self.fig_corrfx  )
        pdf.savefig( self.fig_corrfy  )
        pdf.savefig( self.fig_corrxy  )
        pdf.savefig( self.fig_autocorr  )
        pdf.savefig( self.fig_matrix  )
        pdf.savefig( self.fig_phasefold  )
#            pdf.savefig( self.fig_phasefold_arma  )
        pdf.savefig( self.fig_info_page  )
        pdf.savefig( self.fig_stacked_image  )
        pdf.savefig( self.fig_detrending  )
#            pdf.savefig( self.fig_detrending_over_time )
        print 'Plots saved as ' + outfilename
        
    if self.show_plot == False: plt.close('all'):
        
'''
    

     



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 15 14:04:31 2017

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""



from identify.centroiding import centroid
from identify.centroiding_multi import evaluate_candidate_shortlist



###############################################################################
# Define version
###############################################################################
__version__ = '0.3.5'



def identify(fieldname, obj_id, ngts_version, **kwargs):
    '''
    identify centroid shift for a given NGTS object via identify/centroiding.py.
    see identify/centroiding.py for docstring
    '''
    
    C = centroid(fieldname, obj_id, ngts_version, **kwargs) 
    C.run()



def identify_list(fname, **kwargs):
    '''
    identify centroid shifts for a list of objects
    
    Parameters
    ----------
    fname : string
        path and name of file which includes four columns:
        fieldname   obj_id  ngts_version    source
    dt : float
        precision in phase-folded time series; standard is 0.005
    outdir : str
        path to directory for output; 
        if None a folder called 'output' will be created in the current directory
    mode : str
        'skipfiles': any existing outputs in outdir will be skipped, judging by 'flagfile.txt'
        'newdir': a new outdir will be created with suffix _2
        'replacefiles': any existing outputs will be replaced with new ones
        
    Returns
    -------
    -
    '''
    
    evaluate_candidate_shortlist(fname, **kwargs)





# -*- coding: utf-8 -*-
"""
Created on Tue Oct 18 11:27:46 2016

@author:
Maximilian N. Guenther
Battcock Centre for Experimental Astrophysics,
Cavendish Laboratory,
JJ Thomson Avenue
Cambridge CB3 0HE
Email: mg719@cam.ac.uk
"""

#import warnings
import numpy as np
import matplotlib.pyplot as plt
#from astropy.modeling import models, fitting
from matplotlib.colors import LogNorm
from matplotlib.ticker import MultipleLocator
import os, sys, socket
import glob
import fitsio
#from ngtsio import ngtsio
try:
    from photutils.background import Background
except ImportError:
    from photutils.background import Background2D as Background
from mpl_toolkits.axes_grid1 import make_axes_locatable
#from photutils import daofind
from astropy.stats import sigma_clipped_stats
#from photutils import CircularAperture


from ngtsio import ngtsio
    

def standard_fnames(fieldname, ngts_version, root=None):
    
    if (root is None):
        
        #::: on laptop (OS X)
        if sys.platform == "darwin":
            root = '/Users/mx/Big_Data/BIG_DATA_NGTS/2016/'+ngts_version+'/STACKED_IMAGES/'
                
        #::: on Cambridge servers
        elif 'ra.phy.cam.ac.uk' in socket.gethostname():
            root = '/appch/data/mg719/ngts_pipeline_output/'+ngts_version+'/STACKED_IMAGES/'
            
        #::: on Warwick servers (ngtshead)
        elif 'ngts' in socket.gethostname():
            root = '/home/maxg/ngts_pipeline_output/'+ngts_version+'/STACKED_IMAGES/'
   
    filename = glob.glob( os.path.join( root, 'DITHERED_STACK_'+fieldname+'*.fits' ) )
    
    if len(filename)>0: 
        filename = filename[-1]
    else:
        filename = None
                        
    return filename
        
        
        
def load(fieldname, ngts_version):
    
    filename = standard_fnames(fieldname, ngts_version)
    
    if filename is not None:
        data = fitsio.read( filename )
    else:
        data = None
        
    return data
    
    

def plot(fieldname, ngts_version, x, y, r=15, scale = 4, ax=None, show_apt=True, show_cbar=True, markersize=6):
    '''
        Note: it is important to transform the coordinates!
        
        resolution of stacked images: (8192, 8192)
        resolution of normal CCD images: (2048, 2048)
        -> scaling factor of 4
        
        x and y must additionally be shifted by - 0.5 + 0.5/scale 
        due to miss-match of different image origins in fits vs python vs C
        
        y axis is originally flipped if image is laoded via fitsio.read
    '''
    #::: settings
    x = x - 0.5 + 0.5/scale
    y = y - 0.5 + 0.5/scale
    
    
    #::: load field image
    stacked_image = load(fieldname, ngts_version)
    
    if stacked_image is not None:
        
        #::: cut out stamp/thumbnail
        x0 = np.int( x - r )*scale #*4 as stacked images have 4 x higher resolution than normal images
        x1 = np.int( x + r )*scale
        y0 = np.int( y - r )*scale
        y1 = np.int( y + r )*scale
        stamp = stacked_image[ y0:y1, x0:x1 ] #x and y are switched in indexing
        
        
        #::: plot
        extent=1.*np.array([x0,x1,y0,y1])/scale
        
        if ax is None:
            fig = remove_medianbkg_plot(stamp, x, y, r, scale, extent, show_apt)
    #        fig = remove_bkg_plot(stamp, x, y, r, scale, extent, show_apt)
    
        else:
            simple_plot(stamp, x, y, extent, show_apt, ax, show_cbar, markersize)
            fig = None
    
    else:
        fig = None
        
    return fig
    
    

def simple_plot(stamp, x, y, extent, show_apt, ax, show_cbar, markersize):    
    #::: remove bkg
    try:
        bkg = Background(stamp, (15,15), filter_shape=(3, 3), method='median')
        stamp -= bkg.background
    except:
        pass
    
    #::: plot
    if ax is None:
        fig, ax = plt.subplots(figsize=(6,6))
        
    im = ax.imshow( stamp, norm=LogNorm(vmin=1, vmax=1000), interpolation='nearest', origin='lower', extent=extent )
    ax.set(xlabel='CCDX', ylabel='CCDY')
        
    if show_cbar==True:    
#        cbar = plt.colorbar(im)
#        cbar.set_label('FLUX')
        '''
        to properly show cbar in subplot:
        http://stackoverflow.com/questions/18266642/multiple-imshow-subplots-each-with-colorbar
        '''
        # Create divider for existing axes instance
        divider = make_axes_locatable(ax)
        # Append axes to the right of ax3, with 20% width of ax3
        cax = divider.append_axes("right", size="20%", pad=0.05)
        # Create colorbar in the appended axes
        # Tick locations can be set with the kwarg `ticks`
        # and the format of the ticklabels with kwarg `format`
        cbar = plt.colorbar(im, cax=cax)
        cbar.set_label('FLUX')
    
    if show_apt == True:
        ax.plot( x, y, 'ro', markersize=markersize )
        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
        ax.add_artist(circle1)
    
    
    
    
def remove_medianbkg_plot(z, x, y, r, scale, extent, show_apt=True, vmin=1, vmax=10000):
    mean, median, std = sigma_clipped_stats(z, sigma=3.0, iters=5)   
    bkg = median
    
    
    fig, axes = plt.subplots(1,3,figsize=(16,6))
    
    ax = axes[0]
    im = ax.imshow(z, norm=LogNorm(vmin=vmin, vmax=vmax), origin='lower', extent=extent, cmap='Greys')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="10%", pad=0.05)
    cbar = plt.colorbar(im, cax=cax)
    ax.set(xlabel='CCDX', ylabel='CCDY', title='raw')
    plt.setp(ax.get_xticklabels(), rotation=45)
    
    ax = axes[1]
    im = ax.imshow(z - bkg, norm=LogNorm(vmin=vmin, vmax=vmax), interpolation='nearest', origin='lower', extent=extent, cmap='Greys')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="10%", pad=0.05)
    cbar = plt.colorbar(im, cax=cax)
    ax.set(xlabel='CCDX', title='raw - bkg')
    plt.setp(ax.get_xticklabels(), rotation=45)
   
    if show_apt == True:
        ax.scatter( x, y, c='r' )
        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
        ax.add_artist(circle1)
    
    ax = axes[2]
    rr = scale*r
    cc = scale*5
    image = ( z - bkg )[rr-cc:rr+cc,rr-cc:rr+cc]
    im = ax.imshow(image, norm=LogNorm(vmin=vmin, vmax=vmax), interpolation='nearest', extent=extent+np.array([r-5,-r+5,r-5,-r+5]), origin='lower', cmap='Greys')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="10%", pad=0.05)
    cbar = plt.colorbar(im, cax=cax)
    ax.set(xlabel='CCDX', title='raw - bkg (zoom)')

    plt.setp(ax.get_xticklabels(), rotation=45)
           
    if show_apt == True:
        ax.scatter( x, y, c='r' )
        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
        ax.add_artist(circle1)
        
    plt.tight_layout()
    
    return fig

    
    
    
def remove_bkg_plot(z, x, y, r, scale, extent, show_apt=True, vmin=1, vmax=10000):
    bkg = Background(z, (30,30), filter_shape=(3,3), method='median')
    
    fig, axes = plt.subplots(1,4,figsize=(16,6))
    
    ax = axes[0]
    im = ax.imshow(z, norm=LogNorm(vmin=vmin, vmax=vmax), origin='lower', extent=extent, cmap='Greys')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="10%", pad=0.05)
    cbar = plt.colorbar(im, cax=cax)
    ax.set(xlabel='CCDX', ylabel='CCDY', title='raw')
    plt.setp(ax.get_xticklabels(), rotation=45)
    
    ax = axes[1]
    im = ax.imshow(bkg.background, origin='lower', extent=extent, cmap='Greys')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="10%", pad=0.05)
    cbar = plt.colorbar(im, cax=cax, format="%.1f")
    ax.set(xlabel='CCDX', title='bkg')
    plt.setp(ax.get_xticklabels(), rotation=45)
    
    ax = axes[2]
    im = ax.imshow(z - bkg.background, norm=LogNorm(vmin=vmin, vmax=vmax), interpolation='nearest', origin='lower', extent=extent, cmap='Greys')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="10%", pad=0.05)
    cbar = plt.colorbar(im, cax=cax)
    ax.set(xlabel='CCDX', title='raw - bkg')
    plt.setp(ax.get_xticklabels(), rotation=45)
   
    if show_apt == True:
        ax.scatter( x, y, c='r' )
        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
        ax.add_artist(circle1)
    
    ax = axes[3]
    rr = scale*r
    cc = scale*5
    image = ( z - bkg.background )[rr-cc:rr+cc,rr-cc:rr+cc]
    im = ax.imshow(image, norm=LogNorm(vmin=vmin, vmax=vmax), interpolation='nearest', extent=extent+np.array([r-5,-r+5,r-5,-r+5]), origin='lower', cmap='Greys')
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="10%", pad=0.05)
    cbar = plt.colorbar(im, cax=cax)
    ax.set(xlabel='CCDX', title='raw - bkg (zoom)')
    plt.setp(ax.get_xticklabels(), rotation=45)
           
    if show_apt == True:
        ax.scatter( x, y, c='r' )
        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
        ax.add_artist(circle1)
        
    plt.tight_layout()
    
    return fig




#def fit_Gaussian2D(z):
#    
##    xx, yy = np.meshgrid( np.arange(x0, x1), np.arange(y0, y1) )
#
#    x = np.arange(0,z.shape[0])
#    y = np.arange(0,z.shape[1])
#    xx, yy = np.meshgrid( x,y )
#
#    plt.figure()
#    im = plt.imshow(z, norm=LogNorm(vmin=1, vmax=1000), origin='lower', cmap='Greys')
#    plt.colorbar(im)
#    
#    
#    #::: PHOTUTILS SOURCE DETECTION
#    data = z
#    mean, median, std = sigma_clipped_stats(data, sigma=3.0, iters=5)     
#    sources = daofind(data - median, fwhm=4.0, threshold=4.*std)
#    positions = (sources['xcentroid'], sources['ycentroid'])
#    apertures = CircularAperture(positions, r=3.*4)
##    plt.figure()
##    plt.imshow(data, cmap='Greys', norm=LogNorm(vmin=1, vmax=1000))
#    apertures.plot(color='blue', lw=1.5, alpha=0.5)
#    #:::
#    
#    
#    amplitude_A = np.max( z )
#    x_A_mean = x[-1]/2
#    y_A_mean = y[-1]/2
#    x_A_stddev = 4. #assume FWHM of 1 pixel
#    y_A_stddev = 4.
#    
#    amplitude_B = 0. #assume no second peak
#    x_B_mean = x[-1]/2
#    y_B_mean = y[-1]/2
#    x_B_stddev = 4. #assume FWHM of 1 pixel
#    y_B_stddev = 4.
#    
#    
#    # Now to fit the data create a new superposition with initial
#    # guesses for the parameters:
#    gg_init = models.Gaussian2D(amplitude_A, x_A_mean, y_A_mean, x_A_stddev, y_A_stddev) \
#                + models.Gaussian2D(amplitude_B, x_B_mean, y_B_mean, x_B_stddev, y_B_stddev)
#    fitter = fitting.SLSQPLSQFitter()
#    gg_fit = fitter(gg_init, xx, yy, z)
#    
#    print gg_fit
#    print gg_fit.parameters
#    print gg_fit(xx, yy)
#    print gg_fit(0,0)
#    print gg_fit(60,60)
#    
#    
#    plt.figure()
#    plt.imshow(gg_fit(xx,yy), label='Gaussian')


    
        
        
#def test_plot(fieldname, x, y, r=15, ax=None):
#    '''
#        Note: it is important to transform the coordinates!
#        
#        resolution of stacked images: (8192, 8192)
#        resolution of normal CCD images: (2048, 2048)
#        
#        y axis is originally flipped if image is laoded via fitsio.read
#    '''
#    scale = 4
#    
#    stacked_image = load(fieldname)
#    stacked_image -= 0.95*np.nanmedian(stacked_image) #200
#    
##    fig, ax1 = plt.subplots()
##    ax1.hist( stacked_image.flatten(), bins=np.linspace(200, 240, 50))
#    
#    x0 = np.int( x - r )*scale #*4 as stacked images have 4 x higher resolution than normal images
#    x1 = np.int( x + r )*scale
#    y0 = np.int( y - r )*scale
#    y1 = np.int( y + r )*scale
#    
#    im = stacked_image[ y0:y1, x0:x1 ]
#    
#    if ax is None:
#        fig, axes = plt.subplots(1,3,figsize=(18,6))
#    
#    for i in range(3):
#        if i == 0:
#            x = x
#            y = y
#            title = 'no shift'
#        elif i == 1:
#            x = x - 0.5 + 0.5/scale
#            y = y - 0.5 + 0.5/scale
#            title = 'shift - 3/4 * 0.5'
#        elif i == 2:
#            x = x -0.5
#            y = y -0.5
#            title = 'shift - 0.5'
#        ax = axes[i]
#        
#        ax.imshow( im, norm=LogNorm(vmin=1, vmax=1000), interpolation='nearest', origin='lower', extent=1.*np.array([x0,x1,y0,y1])/scale )
#        ax.scatter( x, y, c='r' )
#        circle1 = plt.Circle((x, y), 3, color='r', fill=False, linewidth=3)
#        ax.add_artist(circle1)
#        ax.set_title(title)
        
        
        
if __name__ == '__main__':
#    fieldname = 'NG0304-1115'
#    ngts_version = 'CYCLE1706'
#    obj_id = 9861
#    obj_id = 992
#    obj_id = 1703
#    obj_id = 6190
#    obj_id = 24503
#    obj_id = 7505
#    obj_id = 16335
    
#    data = ngtsio.get(fieldname, ngts_version, ['CCDX','CCDY'], obj_id=obj_id)
    
    plot(fieldname, np.mean(data['CCDX']), np.mean(data['CCDY']), r=15) #obj 9861
#    plot('NG0304-1115', 1112, 75, r=15)
    
    
